package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;

import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;

public class FastestAlgorithm3 implements Algorithm {

	final Integer INFINITY = Integer.MAX_VALUE;
	
	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}

	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		List<Journey> fastest = new ArrayList<Journey>();
		//Departure agora comea com a primeira conexo de s
		int departure = s.nextConnection(initTime-1);
		int n = graph.getNumOfNodes();
		List <Integer> td = new ArrayList<Integer>(n);
		List <Integer> time = new ArrayList<Integer>(n);
		for (int i = 0; i < n; i++){
			td.add(INFINITY);
			time.add(INFINITY);
		}
		//O caminho de s para s tem custo 0
		td.set(s.getId(), 0);
		time.set(s.getId(), 0);
		
		while (departure >= 0 && departure < INFINITY){
			//Delta comea com o prximo evento em s
			int delta = s.nextConnection(departure) - departure;
			Algorithm foremost = new ForemostAlgorithm();
			List<Journey> journeys = foremost.calculate(graph, s, departure);
			for (int i = 0; i < n; i++){
				int arrival = journeys.get(i).getArrivalDate();
				//s j foi tratado ao se inicializar delta e ns sem jornadas no interessam
				if (journeys.get(i).getSize() == 0)
					continue;
				//Tenta atrasar a jornada (antes do primeiro if)
				journeys.get(i).tryToDelay();
				//Obs: Aps atrasar a jornada, departure  atualizado.
				delta -= (journeys.get(i).getDeparture() - departure);
				if (delta < 1) delta = 1;
				departure = journeys.get(i).getDeparture();
				
				if (arrival - departure < time.get(i)){
					time.set(i, arrival-departure);
					td.set(i, departure);
				}
				//if removido
				int t = graph.getNode(i).nextConnection(arrival);
				if (t != INFINITY && delta > t - arrival)
					delta = t - arrival;
			}
			//TODO ...
			
			departure += delta;
		}
		
		for (int i = 0; i < n; i++){
			Algorithm foremost = new ForemostAlgorithm();
			Journey journey = foremost.calculate(graph, s, td.get(i)).get(i);
			fastest.add(journey);
		}
			
		return fastest;
	}

}
