package br.usp.ime.evolvinggraph;

import java.util.LinkedList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;


public class Schedule {
	private SortedSet<Interval> sched;

	public Schedule(){
		sched = new TreeSet<Interval>();
	}
	
	void add(Integer start, Integer end){
		Interval newInterval = new Interval(start,end);
		List<Interval> affected = new LinkedList<Interval>();
		
		for (Interval currInterval : sched){
			if (!currInterval.isDisjoint(newInterval))
				affected.add(currInterval);
			if (currInterval.start > newInterval.end)
				break;
		}
		
		for (Interval interval : affected){
			sched.remove(interval);
			newInterval.union(interval);
		}
		sched.add(newInterval);
	}
	
	void remove(Integer start, Integer end){
		Interval toRemove = new Interval(start,end);
		List<Interval> affected = new LinkedList<Interval>();
		
		for (Interval currInterval : sched){
			if (!currInterval.isDisjoint(toRemove))
				affected.add(currInterval);
			if (currInterval.start > toRemove.end)
				break;
		}
		
		for (Interval interval : affected){
			sched.remove(interval);
			Interval aux = interval.subtract(toRemove);
			if (interval.isEmpty() == false){
				sched.add(interval);
				if (aux.isEmpty() == false)
					sched.add(aux);
			}
		}
	}
	
	Integer nextTime(int time){
		for (Interval interval : sched){
			if (interval.contains(time))
				return time;
			if (interval.start > time)
				return interval.start;
		}
		return null;
	}
	
	Integer lastTime(int time){
		Integer last = null; 
		for (Interval interval : sched){
			if (interval.contains(time))
				return time;
			last = interval.end - 1;
			if (interval.start > time)
				return last;
		}
		return last;
	}
	
	boolean isEmpty(){
		return sched.isEmpty();
	}

	/**
	 * @return Number of disjoint intervals.
	 */
	Integer size(){
		return sched.size();
	}
	
	Integer last(){
		return sched.last().end;
	}
	
	public Interval getIntervalContaining(Integer time){
		for (Interval interval : sched){
			if (interval.contains(time))
				return interval;
		}
		return null;
	}
	
	public String toString(){
		String str = "";
		for (Interval interval : sched){
			str += " [" + interval.start + "," + interval.end + "["; 
		}
		return str;
	}

	public boolean contains(int t) {
		Interval i = getIntervalContaining(t);
		if (i == null)
			return false;
		return true;
	}

	public int getNextEvent(int arrival) {
		for (Interval i : sched){
			if (arrival < i.start) return i.start;
			if (arrival < i.end) return i.end;
		}
		return Integer.MAX_VALUE;
	}
	
	public SortedSet<Interval> getIntervals(){
		return sched;
	}
}
