#!/usr/bin/perl

#  USAGE: run the script with a .wkt file as first argument and the scaling factor as second argument.
#  The .wkt file needs that words such as "LINESTRING" or "POINT" must be separate from parentheses, 
#  commas and numbers with at least a space. The generated file has the same name as the input file,
#  with "_scaled" as a suffix.

#  Last modified: 18/09/2009 

package Toolkit;
use FileHandle;

my $scale = $ARGV[1];

my $fullFileName = $ARGV[0];
my $newFileName;

if($fullFileName =~ m/\/?(\w+).wkt/){
	$newFileName = ">".$1."_scaled.wkt";
}

open(FILE, $fullFileName) or die "Bad file opening.";
open(FILEOUT, $newFileName) or die "Bad file creation.";

my @matches;

while(<FILE>){
	
	my $line = $_;
	@matches = split(/ /, $line);
		
	my $position = 0;
	foreach my $match (@matches){
		
		if($match =~ m/\((\-?\d+.\d+)/){
			my $number = $1 * $scale;
			print FILEOUT "(".$number." ";
			next;
		}
		elsif($match =~ m/(\-?\d+.\d+)\)/){
			my $number = $1 * $scale;
			print FILEOUT $number.") ";
			next;	
		}
		elsif($match =~ m/(\-?\d+.\d+),/){
			my $number = $1 * $scale;
			print FILEOUT $number.", ";
			next;
		}
		elsif($match =~ m/(\-?\d+.\d+)/){
			my $number = $1 * $scale;
			print FILEOUT $number." ";
			next;
		}
		else{
			print FILEOUT $match." ";
		}
		
	}
	
}

close FILE;
close FILEOUT;
